﻿using HealthMonitor.Core.Common;
using HealthMonitor.Util.Common.Operator;

namespace HealthMonitor.Core.Operator
{
    /// <summary>
    /// 数据库的批量操作缓存管理器
    /// </summary>
    public interface IOperatorManager
    {
        Dictionary<string, OperateCacheItem> PendingInsertOperators { get; }
        Dictionary<string, OperateCacheItem> PendingUpdateOperators { get; }
        Dictionary<string, OperateCacheItem> PendingDeleteOperators { get; }

        /// <summary>
        /// 启动实时模式事件
        /// </summary>
        event Delegates.MonitorModeSetupHandler StartRealtimeMode;

        /// <summary>
        /// 启动悠闲模式事件
        /// </summary>
        event Delegates.MonitorModeSetupHandler StartIdleMode;

        void Append(string key, OperateCacheItem item);

        /// <summary>
        /// 通过查询键，获取待插入的数据
        /// </summary>
        /// <param name="key"></param>
        /// <returns></returns>
        OperateCacheItem? GetDelayOperator(DelayOperateTypeEnum type, string key);

        /// <summary>
        /// 通过查询键，更新指定缓存队列的数据
        /// </summary>
        /// <param name="type"></param>
        /// <param name="key"></param>
        /// <param name="item">新值</param>
        /// <returns>是否更新成功</returns>
        bool ModifyDelayOperator(DelayOperateTypeEnum type, string key, OperateCacheItem item);

        /// <summary>
        /// 通过查询键，删除指定缓存队列的数据
        /// </summary>
        /// <param name="type"></param>
        /// <param name="key"></param>
        void RemoveDelayOperator(DelayOperateTypeEnum type, string key);

        /// <summary>
        /// 获取待插入的数据集
        /// </summary>
        /// <returns></returns>
        Dictionary<string, OperateCacheItem> GetBulkInsertOperators();

        /// <summary>
        /// 获取待更新的数据集
        /// </summary>
        /// <returns></returns>
        Dictionary<string, OperateCacheItem> GetBulkUpdateOperators();

        /// <summary>
        /// 获取待删除的数据集
        /// </summary>
        /// <returns></returns>
        Dictionary<string, OperateCacheItem> GetBulkDeleteOperators();

        /// <summary>
        /// 记录插入失败的操作集
        /// </summary>
        /// <param name="operators"></param>
        void PushFailureInsertOperators(IEnumerable<OperateCacheItem> operators);

        /// <summary>
        /// 记录更新失败的操作集
        /// </summary>
        /// <param name="operators"></param>
        void PushFailureUpdateOperators(IEnumerable<OperateCacheItem> operators);

        /// <summary>
        /// 记录删除失败的操作集
        /// </summary>
        /// <param name="operators"></param>
        void PushFailureDeleteOperators(IEnumerable<OperateCacheItem> operators);

        /// <summary>
        /// 获取插入失败的操作集
        /// </summary>
        /// <param name="count"></param>
        /// <returns></returns>
        IEnumerable<OperateCacheItem> PopFailureInsertOperators(int count = CoreConsts.MAX_FAILURE_OPERATE_POP_COUNT);

        /// <summary>
        /// 获取更新失败的操作集
        /// </summary>
        /// <param name="count"></param>
        /// <returns></returns>
        IEnumerable<OperateCacheItem> PopFailureUpdateOperators(int count = CoreConsts.MAX_FAILURE_OPERATE_POP_COUNT);

        /// <summary>
        /// 获取删除失败的操作集
        /// </summary>
        /// <param name="count"></param>
        /// <returns></returns>
        IEnumerable<OperateCacheItem> PopFailureDeleteOperators(int count = CoreConsts.MAX_FAILURE_OPERATE_POP_COUNT);
    }
}
